function [l,lGrad,N] = H_likelihood(pars,hyperpars,modelType,imageIDs,subIDs,dataset,pars0)
% function [L] = H_likelihood(pars,hyperpars,modelType,imageIDs,subIDs,dataset,pars0)
% calculates the likelihood for a hierarchical model with parameters pars
% for the individual subjects and hyperparameters to model the population
%
% pars = matrix subject x pars (sigma attention and sigma inhibition)
% hyperpars = vector
%       1 mu attention
%       2 mu inhibition
%       3 sigma attention
%       4 sigma inhibtion
%       5 correlation rho
%
% In pars0 you may pass different parameters for all other parameters to be
% used in the simulations.


if ~exist('modelType','var') || isempty(modelType)
    modelType = 2;
end

if ~exist('imageIDs','var') || isempty(imageIDs)
    imageIDs = [1:5,16:20];
end

if ~exist('subIDs','var') || isempty(subIDs)
    subIDs = 1:15;
end
if ~exist('dataset','var') || isempty(dataset)
    dataset = 1;
end
if ~exist('pars0','var') || isempty(pars0)
    pars0 = getMLPars;
    pars0 = exp(pars0([1:8,10]));
end

a = .25;
b = 1;
l = 0;
imMax = length(imageIDs);
N = 0;

options = struct;
switch modelType
    case 1  % Model Engbert et al 2015
        options.targetSelector = 1;
        options.normalizeInhib = true;
        options.nonzero = false;
    case 2  % full model
        options.targetSelector = 3;
        options.normalizeInhib = true;
        options.nonzero = true;
end

lGrad = zeros(size(pars));

%% calculate likelihood

assert(size(pars,1)==length(subIDs),'Exactly one parameter pers subject required')
assert(size(pars,2)==2, 'Expected 2 parameters, sigma Attention and sigma inhibtiion')

if abs(hyperpars(5)>=1) || any(hyperpars(1:4)<=0)
    l = inf;
else
    
    sigmaA = hyperpars(3);
    sigmaI = hyperpars(4);
    covMat = [sigmaA,hyperpars(5).*sqrt(sigmaA.*sigmaI);...
        hyperpars(5).*sqrt(sigmaA.*sigmaI),sigmaI];
    mu = hyperpars(1:2);
    mu = mu(:);
    diff = bsxfun(@minus,pars,mu');
    
    hyperL = -trace(diff*(covMat\diff'))./2-size(diff,1).*(log(2*pi)+log(det(covMat))/2);
    hyperPrior = log(1/2)+invGammaPdf(sigmaA,a,b)+invGammaPdf(sigmaI,a,b);
    
    imCounter = 0;
    for im = imageIDs
        imCounter = imCounter +1;
        [fix,saliency,deg_size] = read_data(dataset,im);
        kSub = 0;
        for sub = subIDs
            kSub = kSub+1;
            if any(fix(:,4)==sub)
                fixSub = fix(fix(:,4)==sub,:);
                times     = fixSub(:,3);% NOT cumsum(fixSub(:,3));
                fixations = fixSub(:,[1,2]);
                pSub = pars0;
                pSub([3,4]) = pars(kSub,:);
                pSub(3) = pSub(3) .* sqrt(pSub(6));
                pSub(4) = pSub(4) .* sqrt(pSub(5));
                [~,likelihood] = dynamic_model_derivative([128,128],deg_size,saliency,fixations(1,:),times/1000,fixations(2:end,:),pSub,[],[],options);
                likelihood(likelihood<=0) = 10.^(-100);
                if any(isnan(likelihood))
                    keyboard
                end
                l        = l+sum(log(likelihood));
                N        = N + size(likelihood,1);
            end
        end
        %l = l+limage/(size(fix(ismember(fix(:,4),subIDs)),1)-length(subIDs));
        %fprintf('finished image %d:%d of %d\n', im,imCounter,imMax);
    end
    
    
    %reverse sign and add prior
    l = -l-hyperL-hyperPrior;
end
end


function logp = invGammaPdf(x,a,b)
% calculates the log-likelihood for the inverse Gammapdf we need for the
% prior on the variances.
%p = b^a./gamma(a).*x.^(-a-1).*exp(-b./x);
logp = a.*log(b)-gammaln(a)-(a+1).*log(x)-b./x;
logp(x<=0) = -inf;
end