function [l,N] = dynamic_likelihood(pars,modelType,imageIDs,subIDs,dataset)
% function [l,N] = dynamic_likelihood(pars,modelType,imageIDs,subIDs,dataset)
% this function calculates the log-likelihood of a given parameter. This
% runs the model on all datasets available and thus obtains a single value
% log-likelihood, which can be used for further calculations like sampling
% etc.
%
% imageIDs, subIDs and dataset specify which images and subjects from which
% dataset to use. The dataset is used as specified in read_data.
%
% The parameter should be given as the logarithm of the original
% parameters.
%


if ~exist('modelType','var') || isempty(modelType)
    modelType = 2;
end

if ~exist('imageIDs','var') || isempty(imageIDs)
    imageIDs = [1:5,16:20];
end

if ~exist('subIDs','var') || isempty(subIDs)
    subIDs = 1:15;
end
if ~exist('dataset','var') || isempty(dataset)
    dataset = 1;
end

l = zeros(numel(imageIDs),numel(subIDs));
N = zeros(numel(imageIDs),numel(subIDs));

pars = pars(:);
options = struct;
switch modelType
    case 1  % Model Engbert et al 2015
        options.normalizeInhib = true;
        options.nonzero = false;
        assert(length(pars)== 8,'Wrong parameter length for desired model')
    case 2  % full model
        options.normalizeInhib = true;
        options.nonzero = true;
        assert(length(pars)== 8,'Wrong parameter length for desired model')
    case 3  % full model both exponents fixed
        options.normalizeInhib = true;
        options.nonzero = true;
        assert(length(pars)== 6,'Wrong parameter length for desired model')
        pars = [pars(1:4);0;0;pars(5:end)];
    case 4  % full model gamma fixed
        options.normalizeInhib = true;
        options.nonzero = true;
        assert(length(pars)== 7,'Wrong parameter length for desired model')
        pars = [pars(1:4);0;pars(5:end)];
    case 5  % full model lambda fixed
        options.normalizeInhib = true;
        options.nonzero = true;
        assert(length(pars)== 7,'Wrong parameter length for desired model')
        pars = [pars(1:5);0;pars(6:end)];
    otherwise
        error('The specified modeltype does not exist!')
end

% parameter transformation
pars = exp(pars);
pars(3) = pars(3) .* sqrt(pars(6));
pars(4) = pars(4) .* sqrt(pars(5));

%% calculate likelihood
imCounter = 0;
for im = imageIDs
    imCounter = imCounter +1;
    [fix,saliency,deg_size] = read_data(dataset,im);
    subCounter = 0;
    for sub = subIDs
        subCounter = subCounter +1;
        if any(fix(:,4)==sub)
            fixSub = fix(fix(:,4)==sub,:);
            times     = fixSub(:,3);% NOT cumsum(fixSub(:,3));
            fixations = fixSub(:,[1,2]);
            [~,likelihood] = dynamic_model([128,128],deg_size,saliency,fixations(1,:),times/1000,fixations(2:end,:),pars,[],[],options);
            likelihood(likelihood<=0) = 10.^(-100);
            if any(~isfinite(likelihood))
                keyboard
            end
            l(imCounter,subCounter) = sum(log(likelihood));
            N(imCounter,subCounter) = size(likelihood,1);
        end
    end
    %l = l+limage/(size(fix(ismember(fix(:,4),subIDs)),1)-length(subIDs));
    %fprintf('finished image %d:%d of %d\n', im,imCounter,imMax);
end

if any(~isfinite(l))
    keyboard
end
